import os.path
import sqlite3


class Data:
    def __enter__(self):
        data_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "Cache", "Face_info.db")
        self.db = sqlite3.connect(data_path)
        self.c = self.db.cursor()

        # 如果不存在则创建一个存DynamicId的表
        self.c.execute("""CREATE TABLE IF NOT EXISTS UID 
                           (ID INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL,
                           Uid  int  NOT NULL,
                           Timestamp int NOT NULL );""")

        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.db.commit()
        self.c.close()
        self.db.close()

    def acquire_uid(self, uid):
        sql = "select Timestamp from UID where Uid=?"
        data = (uid,)
        self.c.execute(sql, data)
        result = self.c.fetchone()
        return result

    def add_uid(self, uid, timestamp):
        sql = "insert into UID (Uid,Timestamp) values(?,?);"
        data = (uid, timestamp,)
        self.c.execute(sql, data)

    def update_timestamp(self, uid, timestamp):
        sql = "update UID set Timestamp = ? where Uid=?"
        data = (timestamp, uid,)
        self.c.execute(sql, data)
