import configparser
import math
import os
import time
from concurrent.futures import ThreadPoolExecutor
from io import BytesIO
from multiprocessing.pool import ThreadPool

import httpx
from PIL import Image, ImageDraw, ImageFont

from .DynamicChecker import Dynamic


class ExtraCardRender:
    def __init__(self, dynamic: Dynamic, isOrigin=False):
        self.font = None
        self.__draw = None
        self.__color = None
        self.__content = None
        self.__container = None
        self.__dynamic = dynamic
        self.__isOrigin = isOrigin
        self.__conf = configparser.ConfigParser()
        self.__base_path = os.path.dirname(os.path.abspath(__file__))
        self.__global_setting()

    def __global_setting(self):
        # 读取配置文件
        self.__conf.read(os.path.join(self.__base_path, "config.ini"))
        # 读取主体背景
        self.__main_color = self.__conf.get("MainContainer", "MainContainerColor")
        # 读取转发背景颜色
        self.__repost_color = self.__conf.get("RepostMainContainer", "MainContainerColor")
        # 读取标题字体
        self.__extra_font = self.__conf.get("ExtraCard", "extra_font")
        # 读取标题颜色
        self.__extra_title_color = self.__conf.get("ExtraCard", "extra_title_color")
        # 读取副标题颜色
        self.__extra_subtitle_color = self.__conf.get("ExtraCard", "extra_subtitle_color")
        # 读取其他颜色
        self.__extra_other_color = self.__conf.get("ExtraCard", "extra_other_color")


        # # extra_card要和背景颜色相反，不然看不清边框
        # if self.__isOrigin:
        #     # 读取背景色
        #     self.__color = self.__conf.get("MainContainer", "MainContainerColor")
        # else:
        #     # 读取背景色
        #     self.__color = self.__conf.get("RepostMainContainer", "MainContainerColor")

    async def extra_article(self):
        if self.__isOrigin:
            cover_url = self.__dynamic.card.origin.image_urls[0]
            title = self.__dynamic.card.origin.title
            summary = self.__dynamic.card.origin.summary
            self.__container = Image.new("RGBA", (1068, 436), self.__repost_color)
        else:
            cover_url = self.__dynamic.card.image_urls[0]
            title = self.__dynamic.card.title
            summary = self.__dynamic.card.summary
            self.__container = Image.new("RGBA", (1068, 436),  self.__main_color)
        self.__content = Image.new("RGBA", (1040, 416), self.__repost_color)
        response = httpx.get(cover_url)
        cover = Image.open(BytesIO(response.content)).convert("RGBA").resize((1040, 240))
        # 容器和主体

        # 贴上封面
        self.__content.paste(cover)
        # 准备写入
        self.__draw = ImageDraw.Draw(self.__content, "RGBA")
        self.font = ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font), 32,
                                       encoding='utf-8')
        # 得到标题的所有字的坐标

        position_list = await self.calculate_text_position(10, 255, 30, 32, 980, 380, title, font_color=self.__extra_title_color)
        # 写入标题
        pool = ThreadPool(5)
        pool.map_async(self.write_pic, position_list)
        pool.close()
        pool.join()
        self.font = ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font), 24,
                                       encoding='utf-8')
        position_list = await self.calculate_text_position(10, 320, 36, 24, 980, 390, summary, True, font_color=self.__extra_title_color)
        # 写入内容
        pool = ThreadPool(5)
        pool.map_async(self.write_pic, position_list)
        pool.close()
        pool.join()
        self.__container.paste(self.__content, (int((self.__container.size[0] - self.__content.size[0]) / 2),
                                                int((self.__container.size[1] - self.__content.size[1]) / 2)))

        return self.__container

    async def calculate_text_position(self, start_x, start_y, y_interval, text_size, x_constraint, y_constrain, text,
                                      interrupt=False, ignor_n=False, font_color=None):
        if interrupt:
            x, y = start_x, start_y
            position_list = []
            break_outer = False
            for i in text:
                if not break_outer:
                    while 1:
                        if i == "\n":
                            if not ignor_n:
                                x = start_x
                                y += y_interval
                                if y > y_constrain:
                                    break_outer = True
                            break
                        else:

                            font_size = self.font.getsize(i)
                            position_list.append(
                                {"content": i, "x": x, "y": y, "font_color": font_color,
                                 "text_size": text_size})
                            x += font_size[0]
                            if x > x_constraint:
                                x = start_x
                                y += y_interval
                                if y > y_constrain:
                                    break_outer = True
                            break
                else:
                    position_list[-1]["content"] = ""
                    position_list[-2]["content"] = "..."
                    break
        else:
            x, y = start_x, start_y
            position_list = []
            break_outer = False
            for i in text:
                if not break_outer:
                    while 1:
                        if i == "\n":
                            break_outer = True
                            break
                        else:

                            font_size = self.font.getsize(i)
                            position_list.append(
                                {"content": i, "x": x, "y": y, "font_color": font_color,
                                 "text_size": text_size})
                            x += font_size[0]
                            if x > x_constraint:
                                break_outer = True
                            break
                else:
                    position_list[-1]["content"] = ""
                    position_list[-2]["content"] = "..."
                    break

        return position_list

    async def picture_render(self):
        """
        对动态中的图片进行渲染
        :return:
        """
        # 图片列表
        pic_list = []
        # 转发动态和一般动态的图片分开获取图片链接和数量
        if self.__isOrigin:
            pictures = self.__dynamic.card.origin.item.pictures
            pictures_count = self.__dynamic.card.origin.item.pictures_count
        else:
            pictures = self.__dynamic.card.item.pictures
            pictures_count = self.__dynamic.card.item.pictures_count
        # 如果图片数量是1
        if pictures_count == 1:
            img_height = pictures[0].img_height
            img_width = pictures[0].img_width
            # 获取1/2大小的图片
            if img_height / img_width > 3:
                img_url = pictures[0].img_src + f"@{img_width}w_{img_width}h_!header.webp"
            else:
                img_url = pictures[0].img_src + f"@{int(img_width / 2)}w.webp"
            response = httpx.get(img_url)
            img = Image.open(BytesIO(response.content))
            # 将图片缩放到和主体一样大
            img = img.resize((1068, int(img.size[1] * 1068 / img.size[0])), Image.ANTIALIAS)
            # 如果是转发动态
            if self.__isOrigin:
                self.__container = Image.new("RGBA", (1068, img.size[1]), self.__main_color)
                self.__container.paste(img, (0, 0))
            else:
                self.__container = Image.new("RGBA", (1068, img.size[1]), self.__repost_color)
                self.__container.paste(img, (int((self.__container.size[0] - img.size[0]) / 2), 0))
            return self.__container
        elif pictures_count in [2, 4]:
            for i in pictures:
                if i.img_height / i.img_width >= 3:
                    img_url = i.img_src + '@250w_250h_!header.webp'
                    pic_list.append(img_url)
                    continue
                img_url = i.img_src + '@250w_250h_1e_1c.webp'
                pic_list.append(img_url)
            with ThreadPoolExecutor(max_workers=4) as pool:
                results = pool.map(self.get_pic, pic_list)
            num = math.ceil(pictures_count / 2)
            if self.__isOrigin:
                self.__container = Image.new("RGBA", (1068, 504 * num + 20), self.__repost_color)
            else:
                self.__container = Image.new("RGBA", (1068, 504 * num + 20), self.__main_color)
            x, y = 20, 10
            for i in results:
                i = i.resize((504, 504), Image.ANTIALIAS)
                self.__container.paste(i, (x, y))
                x += 524
                if x > 1000:
                    x = 20
                    y += 524
            return self.__container
        else:
            for i in pictures:
                if i.img_height / i.img_width >= 3:
                    img_url = i.img_src + '@250w_250h_!header.webp'
                    pic_list.append(img_url)
                    continue
                img_url = i.img_src + '@250w_250h_1e_1c.webp'
                pic_list.append(img_url)
            with ThreadPoolExecutor(max_workers=9) as pool:
                results = pool.map(self.get_pic, pic_list)
            num = math.ceil(pictures_count / 3)
            if self.__isOrigin:
                self.__container = Image.new("RGBA", (1068, 342 * num + 20), self.__repost_color)
            else:
                self.__container = Image.new("RGBA", (1068, 342 * num + 20), self.__main_color)
            x, y = 10, 10
            for i in results:
                i = i.resize((342, 342), Image.ANTIALIAS)
                self.__container.paste(i, (x, y))
                x += 352
                if x > 1000:
                    x = 10
                    y += 352
            return self.__container

    async def extra_music(self):
        if self.__isOrigin:
            title = self.__dynamic.card.origin.title
            typeInfo = self.__dynamic.card.origin.typeInfo
            cover = self.__dynamic.card.origin.cover
            content = Image.new("RGBA", (1040, 160), self.__main_color)
            container = Image.new("RGBA", (1068, 180), self.__repost_color)
        else:
            title = self.__dynamic.card.title
            typeInfo = self.__dynamic.card.typeInfo
            cover = self.__dynamic.card.cover
            content = Image.new("RGBA", (1040, 160), self.__repost_color)
            container = Image.new("RGBA", (1068, 180), self.__main_color)
        response = httpx.get(cover)
        cover = Image.open(BytesIO(response.content)).convert("RGBA").resize((160, 160))
        # 容器
        draw = ImageDraw.Draw(content)
        # 播放图标
        play = Image.open(os.path.join(self.__base_path, "Static", "Picture", "play2.png")).convert("RGBA")
        draw.text((180, 30), title,
                  font=ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font), 28),
                  fill=self.__extra_title_color)
        draw.text((180, 100), typeInfo,
                  font=ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font), 24),
                  fill=self.__extra_title_color)
        content.paste(cover)
        content.paste(play, (30, 30), play)
        container.paste(content, (
            int((container.size[0] - content.size[0]) / 2),
            int((container.size[1] - content.size[1]) / 2)))
        return container

    async def multi_biz_type(self):
        if self.__isOrigin:
            if self.__dynamic.card.origin.sketch.biz_type:
                process_func = {"3": self.extra_decoration, "131": self.extra_song_list, "141": self.extra_guild,
                                "201": self.extra_manga, "231": self.extra_pendant}[
                    str(self.__dynamic.card.origin.sketch.biz_type)]
            else:
                process_func = self.extra_decoration
        else:
            if self.__dynamic.card.sketch.biz_type:
                process_func = {"3": self.extra_decoration, "131": self.extra_song_list, "141": self.extra_guild,
                                "201": self.extra_manga, "231": self.extra_pendant}[
                    str(self.__dynamic.card.sketch.biz_type)]
            else:
                process_func = self.extra_decoration
        img = await process_func()
        return img

    async def extra_decoration(self):
        if not self.__isOrigin:
            title = self.__dynamic.card.sketch.title
            desc_text = self.__dynamic.card.sketch.desc_text
            cover_url = self.__dynamic.card.sketch.cover_url
            content = Image.new("RGBA", (1068, 160), self.__main_color)
            container = Image.new("RGBA", (1068, 160), self.__repost_color)
        else:
            title = self.__dynamic.card.origin.sketch.title
            desc_text = self.__dynamic.card.origin.sketch.desc_text
            cover_url = self.__dynamic.card.origin.sketch.cover_url
            content = Image.new("RGBA", (1040, 160), self.__repost_color)
            container = Image.new("RGBA", (1068, 180), self.__main_color)
        response = httpx.get(cover_url)
        cover = Image.open(BytesIO(response.content)).convert("RGBA").resize((160, 160))
        # 容器
        draw = ImageDraw.Draw(content)
        draw.text((180, 40), title,
                  font=ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font), 20),
                  fill=self.__extra_title_color)
        draw.text((180, 100), desc_text,
                  font=ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font), 15),
                  fill=self.__extra_title_color)
        content.paste(cover)
        container.paste(content, (
            int((container.size[0] - content.size[0]) / 2),
            int((container.size[1] - content.size[1]) / 2)))
        return container

    async def extra_song_list(self):
        """
        处理type 2049
        biz_type 131
        :return:
        """
        if self.__isOrigin:
            title = self.__dynamic.card.origin.sketch.title
            cover_url = self.__dynamic.card.origin.sketch.cover_url
            content = Image.new("RGBA", (1040, 160), self.__main_color)
            container = Image.new("RGBA", (1068, 180), self.__repost_color)
        else:
            title = self.__dynamic.card.sketch.title
            cover_url = self.__dynamic.card.sketch.cover_url
            content = Image.new("RGBA", (1040, 160), self.__repost_color)
            container = Image.new("RGBA", (1068, 180), self.__main_color)

        response = httpx.get(cover_url)
        cover = Image.open(BytesIO(response.content)).convert("RGBA").resize((160, 160))
        # 容器
        self.font = ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font), 28,
                                       encoding='utf-8')
        position = await self.calculate_text_position(start_x=239, start_y=60, y_interval=30, text_size=28,
                                                      x_constraint=800, y_constrain=90, text=title, ignor_n=True,
                                                      font_color=self.__extra_title_color)
        btn = Image.open(os.path.join(self.__base_path, "Static", "Picture", "gd.png")).convert("RGBA")
        btn = btn.resize((int(btn.size[0] / 2.7), int(btn.size[1] / 2.7)))
        draw = ImageDraw.Draw(content)
        for i in position:
            draw.text((i["x"], i["y"]), i["content"],
                      font=ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font),
                                              i["text_size"]), fill=self.__extra_title_color)
        content.paste(cover)
        content.paste(btn, (910, 60), btn)
        container.paste(content, (int((container.size[0] - content.size[0]) / 2),
                                  (int((container.size[1] - content.size[1]) / 2))))
        return container

    async def extra_guild(self):
        """
        处理type 2049
        biz_type 141
        :return:
        """
        if self.__isOrigin:
            title = self.__dynamic.card.origin.sketch.title
            desc_text = self.__dynamic.card.origin.sketch.desc_text
            cover_url = self.__dynamic.card.origin.sketch.cover_url
            content = Image.new("RGBA", (1040, 160), self.__main_color)
            container = Image.new("RGBA", (1068, 180), self.__repost_color)
        else:
            title = self.__dynamic.card.sketch.title
            desc_text = self.__dynamic.card.sketch.desc_text
            cover_url = self.__dynamic.card.sketch.cover_url
            content = Image.new("RGBA", (1040, 160), self.__repost_color)
            container = Image.new("RGBA", (1068, 180), self.__main_color)

        response = httpx.get(cover_url)
        cover = Image.open(BytesIO(response.content)).convert("RGBA").resize((160, 160))
        # 容器
        btn = Image.open(os.path.join(self.__base_path, "Static", "Picture", "guild.png")).convert("RGBA")
        btn = btn.resize((96, 50))
        self.font = ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font), 28,
                                       encoding='utf-8')
        position = await self.calculate_text_position(start_x=240, start_y=30, y_interval=30, text_size=28,
                                                      x_constraint=800, y_constrain=90, text=title, ignor_n=True,
                                                      font_color=self.__extra_title_color)
        draw = ImageDraw.Draw(content)
        for i in position:
            draw.text((i["x"], i["y"]), i["content"], font=self.font, fill=self.__extra_title_color)
        draw.text((239, 100), desc_text, fill=self.__extra_title_color,
                  font=ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font), 20))
        content.paste(cover, (0, 0), cover)
        content.paste(btn, (910, 60), btn)
        container.paste(content, (
            int((container.size[0] - content.size[0]) / 2),
            int((container.size[1] - content.size[1]) / 2)))
        return container

    async def extra_pendant(self):
        if not self.__isOrigin:
            title = self.__dynamic.card.sketch.title
            desc_text = self.__dynamic.card.sketch.desc_text
            cover_url = self.__dynamic.card.sketch.cover_url
            content = Image.new("RGBA", (1068, 160), self.__main_color)
            container = Image.new("RGBA", (1068, 160), self.__repost_color)
        else:
            title = self.__dynamic.card.origin.sketch.title
            desc_text = self.__dynamic.card.origin.sketch.desc_text
            cover_url = self.__dynamic.card.origin.sketch.cover_url
            content = Image.new("RGBA", (1040, 160), self.__repost_color)
            container = Image.new("RGBA", (1068, 180), self.__main_color)
        response = httpx.get(cover_url)
        cover = Image.open(BytesIO(response.content)).convert("RGBA").resize((160, 160))
        # 容器
        draw = ImageDraw.Draw(content)
        # 播放图标
        btn = Image.open(os.path.join(self.__base_path, "Static", "Picture", "zb.png")).convert("RGBA")

        btn = btn.resize((96, 50))
        draw.text((230, 40), title,
                  font=ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font), 28),
                  fill=self.__extra_title_color)
        draw.text((230, 100), desc_text,
                  font=ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font), 24),
                  fill=self.__extra_subtitle_color)
        content.paste(cover)
        content.paste(btn, (910, 60), btn)
        container.paste(content, (int((container.size[0] - content.size[0]) / 2),
                                  (int((container.size[1] - content.size[1]) / 2))))

        return container

    async def extra_manga(self):
        """
        处理type 2049
        biz_type 201
        :return:
        """
        await self.__global_setting()
        if not self.__isOrigin:
            title = self.__dynamic.card.sketch.title
            desc_text = self.__dynamic.card.sketch.desc_text
            text = self.__dynamic.card.sketch.text
            cover_url = self.__dynamic.card.sketch.cover_url
            content = Image.new("RGBA", (1068, 320), self.__main_color)
            container = Image.new("RGBA", (1068, 320), self.__repost_color)
        else:
            title = self.__dynamic.card.origin.sketch.title
            desc_text = self.__dynamic.card.origin.sketch.desc_text
            text = self.__dynamic.card.origin.sketch.text
            cover_url = self.__dynamic.card.origin.sketch.cover_url
            content = Image.new("RGBA", (1040, 320), self.__repost_color)
            container = Image.new("RGBA", (1068, 340), self.__repost_color)
        response = httpx.get(cover_url)
        cover = Image.open(BytesIO(response.content)).convert("RGBA").resize((240, 320))
        # 容器
        content.paste(cover)
        draw = ImageDraw.Draw(content)
        draw.text((260, 40), title,
                  font=ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font), 30),
                  fill=self.__extra_title_color)
        draw.text((260, 100), desc_text,
                  font=ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font), 20),
                  fill=self.__extra_title_color)
        draw.text((260, 150), text,
                  font=ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font), 20),
                  fill=self.__extra_title_color)

        container.paste(content, (int((container.size[0] - content.size[0]) / 2),
                                  (int((container.size[1] - content.size[1]) / 2))))
        return container

    async def extra_repost_live(self, flag=False):
        if not flag:
            cover = self.__dynamic.card.origin.cover
            title = self.__dynamic.card.origin.title
            desc = self.__dynamic.card.origin.area_v2_name + " • " + self.__dynamic.card.origin.watched_show
        else:
            cover = self.__dynamic.card.origin.live_play_info.cover
            title = self.__dynamic.card.origin.live_play_info.title
            desc = self.__dynamic.card.origin.live_play_info.area_name + " • " + self.__dynamic.card.origin.live_play_info.watched_show

        self.__content = Image.new("RGBA", (1040, 254), self.__main_color)
        self.__container = Image.new("RGBA", (1068, 274), self.__repost_color)
        response = httpx.get(cover)
        cover = Image.open(BytesIO(response.content)).convert("RGBA")
        cover = cover.resize((406, 254))
        self.__content.paste(cover)
        self.__draw = ImageDraw.Draw(self.__content, "RGBA")
        self.font = ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font), 28,
                                       encoding='utf-8')
        position_list = await self.calculate_text_position(cover.size[0] + 20, 20, 30, 28, 920, 50, title,
                                                           interrupt=False, font_color=self.__extra_title_color)
        pool = ThreadPool(5)
        pool.map(self.write_pic, position_list)
        pool.close()
        pool.join()
        self.__draw.text((cover.size[0] + 20, 150), desc, fill=self.__extra_title_color,
                         font=ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font), 20))
        self.__content.paste(cover)
        tg = Image.open(os.path.join(self.__base_path, "Static", "Picture", "live.png")).convert("RGBA")
        tg = tg.resize((96, 36))
        self.__content.paste(tg, (280, 20), tg)

        self.__container.paste(self.__content, (int((self.__container.size[0] - self.__content.size[0]) / 2),
                                                int((self.__container.size[1] - self.__content.size[1]) / 2)))

        return self.__container

    async def video_card_render(self):
        if not self.__isOrigin:
            title = self.__dynamic.card.title
            desc = self.__dynamic.card.desc
            pic = self.__dynamic.card.pic
            self.__content = Image.new("RGBA", (1040, 254), self.__repost_color)
        else:
            title = self.__dynamic.card.origin.title
            desc = self.__dynamic.card.origin.desc
            pic = self.__dynamic.card.origin.pic
            self.__content = Image.new("RGBA", (1040, 254), self.__main_color)
            self.__container = Image.new("RGBA", (1068, 274),  self.__repost_color)
        response = httpx.get(pic)
        cover = Image.open(BytesIO(response.content)).convert("RGBA")
        cover = cover.resize((406, 254))
        self.__content.paste(cover)
        tg = Image.open(os.path.join(self.__base_path, "Static", "Picture", "tg.png")).convert("RGBA")
        tg = tg.resize((96, 36))
        self.__content.paste(tg, (280, 20), tg)

        self.__draw = ImageDraw.Draw(self.__content, "RGBA")
        # 标题的所有字的坐标
        self.font = ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font), 28,
                                       encoding='utf-8')
        position_list = await self.calculate_text_position(cover.size[0] + 20, 20, 30, 25, 920, 50, title,
                                                           interrupt=True, font_color=self.__extra_title_color)
        pool = ThreadPool(5)
        pool.map(self.write_pic, position_list)
        pool.close()
        pool.join()
        self.font = ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font), 28,
                                       encoding='utf-8')
        position_list_2 = await self.calculate_text_position(cover.size[0] + 20, 100, 30, 24, 920, 150, desc,
                                                             interrupt=True, ignor_n=True, font_color=self.__extra_other_color)
        # 写入内容
        pool = ThreadPool(5)
        pool.map(self.write_pic, position_list_2)
        pool.close()
        pool.join()
        if self.__isOrigin:
            self.__container.paste(self.__content, (
                int((self.__container.size[0] - self.__content.size[0]) / 2),
                int((self.__container.size[1] - self.__content.size[1]) / 2)))
            return self.__container
        else:
            return self.__content

    async def extra_movie(self):
        new_desc = self.__dynamic.card.origin.new_desc
        if not new_desc:
            new_desc = self.__dynamic.card.origin.apiSeasonInfo.title
        index_title = self.__dynamic.card.origin.index_title
        cover = self.__dynamic.card.origin.cover
        self.__content = Image.new("RGBA", (1028, 254),self.__main_color)
        self.__container = Image.new("RGBA", (1068, 274), self.__repost_color)
        response = httpx.get(cover)
        cover = Image.open(BytesIO(response.content)).convert("RGBA")
        cover = cover.resize((406, 254))
        self.__content.paste(cover)
        self.__draw = ImageDraw.Draw(self.__content, "RGBA")
        self.__draw.text((cover.size[0] + 40, 10), new_desc, fill=self.__extra_title_color,
                         font=ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font), 25))
        self.__draw.text((cover.size[0] + 40, 150), index_title, fill=self.__extra_title_color,
                         font=ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font), 20))
        tg_name = {"1": "fj.png", "5": "dsj.png", "3": "jlp.png", "2": "dy.png"}[
            str(self.__dynamic.card.origin.apiSeasonInfo.bgm_type)]
        tg = Image.open(os.path.join(self.__base_path, "Static", "Picture", tg_name)).convert("RGBA")
        tg = tg.resize((112, 40))
        self.__content.paste(tg, (280, 20), tg)
        self.__container.paste(self.__content, (int((self.__container.size[0] - self.__content.size[0]) / 2),
                                                int((self.__container.size[1] - self.__content.size[1]) / 2)))

        return self.__container

    async def extra_course(self):
        cover = self.__dynamic.card.origin.cover
        title = self.__dynamic.card.origin.title
        subtitle = self.__dynamic.card.origin.subtitle
        update_info = self.__dynamic.card.origin.update_info
        self.__content = Image.new("RGBA", (1028, 254), self.__main_color)
        self.__container = Image.new("RGBA", (1068, 274), self.__repost_color)
        response = httpx.get(cover)
        cover = Image.open(BytesIO(response.content)).convert("RGBA")
        cover = cover.resize((406, 254))
        self.__content.paste(cover)
        self.__draw = ImageDraw.Draw(self.__content, "RGBA")
        self.font = ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font), 24,
                                       encoding='utf-8')
        subtitle_position = await self.calculate_text_position(cover.size[0] + 20, 160, 20, 24, 940, 230, subtitle,
                                                               interrupt=True, ignor_n=True, font_color=self.__extra_title_color)
        pool = ThreadPool(5)
        pool.map(self.write_pic, subtitle_position)
        pool.close()
        pool.join()
        tg = Image.open(os.path.join(self.__base_path, "Static", "Picture", "course.png")).convert("RGBA")
        tg = tg.resize((int(tg.size[0] / 2.8), int(tg.size[1] / 2.8)))
        self.__content.paste(tg, (250, 15), tg)
        self.__draw.text((cover.size[0] + 20, 10), title, fill="black",
                         font=ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font), 28))
        self.__draw.text((cover.size[0] + 20, 80), update_info, fill="black",
                         font=ImageFont.truetype(os.path.join(self.__base_path, "Static", "Font", self.__extra_font), 20))

        self.__container.paste(self.__content, (int((self.__container.size[0] - self.__content.size[0]) / 2),
                                                int((self.__container.size[1] - self.__content.size[1]) / 2)))

        return self.__container

    def write_pic(self, info_dic):

        self.__draw.text((info_dic["x"], info_dic["y"]), info_dic["content"], font=self.font,
                         fill=info_dic["font_color"])

    def get_pic(self, url):
        response = httpx.get(url)
        img = Image.open(BytesIO(response.content))
        return img

    async def isEmoji(self, content):

        if not content:
            return False
        if u"\U0001F600" <= content and content <= u"\U0001F64F":
            return True
        elif u"\U0001F300" <= content and content <= u"\U0001F5FF":
            return True
        elif u"\U0001F680" <= content and content <= u"\U0001F6FF":
            return True
        elif u"\U0001F1E0" <= content and content <= u"\U0001F1FF":
            return True
        else:
            return False
