import configparser
import os
import re
import time
from concurrent.futures import ThreadPoolExecutor
from io import BytesIO
from multiprocessing.pool import ThreadPool

import emoji
import httpx
from PIL import Image, ImageDraw, ImageFont
from fontTools.ttLib.ttFont import TTFont


class MainTextRender:
    def __init__(self, dynamic):

        self.standby_font = None
        self.font_color = None
        self.text_font = None
        self.font_size = None
        self.emoji_font = None
        self.emoji_font_size = None
        self.content_inside = None
        self.dynamic = dynamic
        self.content_img = None
        self.draw = None
        self.font_key = None
        self.emoji_list = []
        self.conf = configparser.ConfigParser()
        self.__base_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "Static")
        self.Emoji_cache_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "Cache", "Emoji")
        # 0 视频 1超链接 2抽奖  3专栏 4恰饭文本
        self.rich_text_Instead = ["㉠", "㉡", "㉣", "㉢", "㉤", "㉥", "㉦"]
        self.emoji_pic_tag_instead = ["㉧", "㉨", "㉩", "㉪", "㉫", "㉬", "㉭", "㉮", "㉯", '㉰', "㉱", "㉲", "㉳", "㉴", "㉵", "㉶",
                                      "㉷", "㉸", "㉹", "㉺", "㉻", "㉿", "㋐", "㋑", "㋒", "㋓", "㋔", "㋕", "㋖", "㋗", "㋘", "㋙",
                                      "㋚", "㋛", "㋜", "㋝", "㋞", "㋟", "㋠", "㋡", "㋢", "㋣", "㋤", "㋥", "㋦", "㋧", "㋨", "㋩"]

    async def text_content_render(self):
        # 读取配置文件
        self.conf.read(os.path.join(os.path.dirname(self.__base_path), "config.ini"))
        # 读取字体大小
        font_size = self.conf.getint("TextRender", "main_font_size")
        # 读取字体颜色
        font_color = self.conf.get("TextRender", "font_color")
        # 读取字体名
        font_name = self.conf.get("TextRender", "main_font")
        # 读取背景色
        text_container_color = self.conf.get("MainContainer", "MainContainerColor")
        # 读取emoji字体大小
        emoji_font_size = self.conf.getint("TextRender", "emoji_font_size")
        # 读取emoji字体名
        emoji_font_name = self.conf.get("TextRender", "emoji_font")
        # 读取备用字体名
        standby_font_name = self.conf.get("TextRender", "standby_font")
        # 设置文字主体文字信息
        self.text_font = ImageFont.truetype(os.path.join(self.__base_path, "Font", font_name), font_size)
        # 设置备用字体
        self.standby_font = ImageFont.truetype(os.path.join(self.__base_path, "Font", standby_font_name), font_size)
        # 设置emoji字体
        self.emoji_font = ImageFont.truetype(os.path.join(self.__base_path, "Font", emoji_font_name), emoji_font_size)
        #
        self.font_key = TTFont(os.path.join(self.__base_path, "Font", font_name), fontNumber=0)['cmap'].tables[
            0].ttFont.getBestCmap().keys()
        # 取出文本主体

        dynamic_type = self.dynamic.desc.type
        if dynamic_type in [1, 4]:
            description = self.dynamic.card.item.content
        elif dynamic_type in [2048, 2049]:
            description = self.dynamic.card.vest.content
        elif dynamic_type == 2:
            description = self.dynamic.card.item.description
        elif dynamic_type == 8:
            description = self.dynamic.card.dynamic
        elif dynamic_type == 256:
            description = self.dynamic.card.intro
        else:
            description = None
        # 如果文本不为空
        if description:
            # 取出ctrl内容
            ctrl = self.dynamic.extend_json.ctrl
            # if dynamic_type in [1, 4]:
            #     ctrl = self.dynamic.card.item.ctrl
            # elif dynamic_type in [2]:
            #     ctrl = self.dynamic.card.item.at_control
            # elif dynamic_type == 8:
            #     ctrl = self.dynamic.extend_json.ctrl
            # else:
            #     ctrl = None
                # 获取文本中所有的emoji
            all_emoji = emoji.emoji_lis(description)
            # 如果emoji列表不为空
            if all_emoji:
                # 将所有emoji内容放进列表内
                for i in all_emoji:
                    self.emoji_list.append(i["emoji"])
            # 通过ctrl内的坐标取出文本主体中的ctrl内容以及被处理好的文本主体
            result = await self.get_particular_text(description=description, at_control=ctrl)
            # 取出被处理好的文本主体
            description = result["content"]
            # 取出所有需要变成蓝色的文本
            heterochrosis_info_list = result["heterochrosis_info_list"]
            # 取出bili_emoji图片列表
            emoji_list = result["emoji_img_list"]
            # 计算所有需要变色的文本的在主体文本中的索引
            particular_text_index_set = await self.calculate_particular_text_index(heterochrosis_info_list, description)
            # 计算所有文字的坐标
            position_info = await self.calculate_position(start_x=20, start_y=10, x_restrain=1078, content=description,
                                                          line_height=42, font_size=font_size,
                                                          particular_text_index_set=particular_text_index_set,
                                                          font_color=font_color,
                                                          img_list=emoji_list)
            # 计算承载文本主体的图片的高
            content_y = position_info[-1]["position"][1] + 50
            # 创建承载文本主体的图片
            self.content_img = Image.new("RGBA", (1108, content_y), color=text_container_color)
            self.draw = ImageDraw.Draw(self.content_img)
            with ThreadPoolExecutor(10) as pool:
                pool.map(self.draw_pic, position_info)
            return self.content_img
        else:
            if self.dynamic.display and self.dynamic.display.topic_info.new_topic:
                description = ""
                result = await self.get_particular_text(description=description)
                description = result["content"]
                heterochrosis_info_list = result["heterochrosis_info_list"]
                emoji_list = result["emoji_img_list"]
                particular_text_index_set = await self.calculate_particular_text_index(heterochrosis_info_list,
                                                                                       description)
                position_info = await self.calculate_position(start_x=20, start_y=10, x_restrain=1078,
                                                              content=description,
                                                              line_height=42, font_size=font_size,
                                                              particular_text_index_set=particular_text_index_set,
                                                              font_color=font_color,
                                                              img_list=emoji_list)
                content_y = position_info[-1]["position"][1] + 50
                self.content_img = Image.new("RGBA", (1108, content_y), color=text_container_color)
                self.draw = ImageDraw.Draw(self.content_img)
                with ThreadPoolExecutor(10) as pool:
                    pool.map(self.draw_pic, position_info)
                return self.content_img
            else:
                return

    async def origin_content_render(self):
        # 读取配置文件
        self.conf.read(os.path.join(os.path.dirname(self.__base_path), "config.ini"))
        # 读取字体大小
        font_size = self.conf.getint("OriginTextRender", "Origin_font_size")
        # 读取字体颜色
        font_color = self.conf.get("OriginTextRender", "Origin_font_color")
        # 读取字体名
        font_name = self.conf.get("OriginTextRender", "Origin_font")
        # 读取背景色
        text_container_color = self.conf.get("RepostMainContainer", "MainContainerColor")
        # 读取emoji字体大小
        emoji_font_size = self.conf.getint("OriginTextRender", "Origin_emoji_font_size")
        # 读取emoji字体名
        emoji_font_name = self.conf.get("OriginTextRender", "Origin_emoji_font")
        # 读取备用字体名
        standby_font_name = self.conf.get("TextRender", "standby_font")
        # 设置文字主体文字信息
        self.text_font = ImageFont.truetype(os.path.join(self.__base_path, "Font", font_name), font_size)
        # 设置emoji字体
        self.emoji_font = ImageFont.truetype(os.path.join(self.__base_path, "Font", emoji_font_name), emoji_font_size)
        # 设置备用字体
        self.standby_font = ImageFont.truetype(os.path.join(self.__base_path, "Font", standby_font_name), font_size)
        self.font_key = TTFont(os.path.join(self.__base_path, "Font", font_name), fontNumber=0)['cmap'].tables[
            0].ttFont.getBestCmap().keys()
        dynamic_type = self.dynamic.desc.orig_type
        if dynamic_type in [1, 4]:
            description = self.dynamic.card.origin.item.content
        elif dynamic_type in [2048, 2049]:
            description = self.dynamic.card.origin.vest.content
        elif dynamic_type == 2:
            description = self.dynamic.card.origin.item.description
        elif dynamic_type == 8:
            description = self.dynamic.card.origin.dynamic
        elif dynamic_type == 256:
            description = self.dynamic.card.origin.intro
        else:
            description = None
        if description:
            ctrl = self.dynamic.card.origin_extend_json.ctrl
            # if dynamic_type in [1, 4]:
            #     ctrl = self.dynamic.card.origin.item.ctrl
            # elif dynamic_type in [2]:
            #     ctrl = self.dynamic.card.origin.item.at_control
            # elif dynamic_type == 8:
            #     ctrl = self.dynamic.card.origin_extend_json.ctrl
            # else:
            #     ctrl = None

            all_emoji = emoji.emoji_lis(description)
            if all_emoji:
                for i in all_emoji:
                    self.emoji_list.append(i["emoji"])
            result = await self.get_origin_particular_text(description=description, at_control=ctrl)
            description = result["content"]

            heterochrosis_info_list = result["heterochrosis_info_list"]
            emoji_list = result["emoji_img_list"]
            particular_text_index_set = await self.calculate_particular_text_index(heterochrosis_info_list, description)
            position_info = await self.calculate_position(start_x=10, start_y=10, x_restrain=1044, content=description,
                                                          line_height=42, font_size=font_size,
                                                          particular_text_index_set=particular_text_index_set,
                                                          img_list=emoji_list, font_color=font_color)

            content_y = position_info[-1]["position"][1] + 50
            self.content_img = Image.new("RGBA", (1068, content_y), color=text_container_color)
            self.draw = ImageDraw.Draw(self.content_img)
            with ThreadPoolExecutor(10) as pool:
                pool.map(self.draw_pic, position_info)
            return self.content_img
        else:
            if self.dynamic.display and self.dynamic.display.origin and self.dynamic.display.origin.topic_info:
                if self.dynamic.display.origin.topic_info.new_topic:
                    description = ""
                    result = await self.get_origin_particular_text(description=description)
                    description = result["content"]
                    heterochrosis_info_list = result["heterochrosis_info_list"]
                    emoji_list = result["emoji_img_list"]
                    particular_text_index_set = await self.calculate_particular_text_index(heterochrosis_info_list,
                                                                                           description)
                    position_info = await self.calculate_position(start_x=10, start_y=10, x_restrain=1044,
                                                                  content=description,
                                                                  line_height=42, font_size=font_size,
                                                                  particular_text_index_set=particular_text_index_set,
                                                                  img_list=emoji_list, font_color=font_color)

                    content_y = position_info[-1]["position"][1] + 50
                    self.content_img = Image.new("RGBA", (1068, content_y), color=text_container_color)
                    self.draw = ImageDraw.Draw(self.content_img)
                    with ThreadPoolExecutor(10) as pool:
                        pool.map(self.draw_pic, position_info)
                    return self.content_img
                else:
                    return
            else:
                return

    async def get_origin_particular_text(self, description=None, at_control=None):
        """
               获取所有蓝色的文本
               :param description: 文本主体
               :param at_control: ctrl列表
               :return:
               """
        info_list = []
        emoji_list = []
        # 取出所有的ctrl字符
        if at_control:
            # 过滤掉所有的emoji以防止其造成ctrl错位
            content = await self.filter_emoji(description)
            for at_detail in at_control:
                at_location = at_detail.location
                at_length = at_detail.length
                at_type = at_detail.type
                if at_type == 1:
                    at_start = at_location
                    at_end = at_start + at_length
                    at_content = content[at_start:at_end]
                    info_list.append({"origin_text": at_content, "at_type": 1, "text_type": "at"})
                elif at_type == 3:
                    at_start = at_detail.location
                    at_end = at_start + int(at_detail.data)
                    at_content = content[at_start:at_end]
                    info_list.append(
                        {"origin_text": at_content, "new_content": self.rich_text_Instead[5] + at_content,
                         "at_type": at_type,
                         "text_type": "at"})

                else:
                    at_start = at_detail.location
                    at_end = at_start + int(at_detail.data)
                    at_content = content[at_start:at_end]
                    info_list.append(
                        {"origin_text": at_content, "new_content": self.rich_text_Instead[at_type] + at_content,
                         "at_type": at_type,
                         "text_type": "at"})

            for info in info_list:
                if info["at_type"] != 1:
                    description = description.replace(info["origin_text"], info["new_content"])

        # 取出所有富文本字符并且把原字符换成富文本字符
        if self.dynamic.display and self.dynamic.display.origin and self.dynamic.display.origin.rich_text:
            for rich_detail in self.dynamic.display.origin.rich_text.rich_details:
                if rich_detail.icon_type == 1:
                    description = description.replace(rich_detail.orig_text,
                                                      self.rich_text_Instead[0] + rich_detail.text)
                    info_list.append({"origin_text": rich_detail.text, "text_type": "rich_text"})
                else:
                    description = description.replace(rich_detail.orig_text,
                                                      self.rich_text_Instead[3] + rich_detail.text)
                    info_list.append({"origin_text": rich_detail.text, "text_type": "rich_text"})
        # 把所有的bili_emoji换成特殊字符，并且获取所有的bili_emoji图片
        if self.dynamic.display and self.dynamic.display.origin and self.dynamic.display.origin.emoji_info:
            emoji_url_list = []
            for i in range(len(self.dynamic.display.origin.emoji_info.emoji_details)):
                emoji_details = self.dynamic.display.origin.emoji_info.emoji_details
                description = description.replace(emoji_details[i].emoji_name, self.emoji_pic_tag_instead[i])
                info_list.append(
                    {self.emoji_pic_tag_instead[i]: emoji_details[i].emoji_name, "text_type": "bili_emoji"})
                emoji_url_list.append({"url": emoji_details[i].url, "name": emoji_details[i].emoji_name})
            with ThreadPoolExecutor(max_workers=5) as pool:
                results = pool.map(self.get_emoji, emoji_url_list)
                for i in results:
                    emoji_list.append(i)
        # 获取所有的话题标签
        if self.dynamic.display and self.dynamic.display.origin and self.dynamic.display.origin.topic_info:
            if self.dynamic.display.origin.topic_info.topic_details:
                for topic_detail in self.dynamic.display.origin.topic_info.topic_details:
                    info_list.append({"origin_text": f"#{topic_detail.topic_name}#", "text_type": "topic"})
            if self.dynamic.display.origin.topic_info.new_topic:
                topic_name = self.dynamic.display.origin.topic_info.new_topic.name
                description = self.rich_text_Instead[6] + topic_name + "\n" + description
                info_list.append({"origin_text": self.rich_text_Instead[6] + topic_name, "text_type": "new_topic"})

        # 去除文本主体中的特殊符号
        description = description.replace("\r", "")
        if self.emoji_list:
            temp = []
            for i in self.emoji_list:
                list_info = list(i)
                if len(list_info) != 1:
                    emoji_first = list_info[0]
                    description = description.replace(i, emoji_first)
                    temp.append(emoji_first)
                    continue
                temp.append(i)
            self.emoji_list = temp

        # region
        # 获取文本中所有的url
        reg = r'https?:[0-9a-zA-Z.?#&=_@(-/\d]+'
        urls = re.findall(reg, description)
        for url in urls:
            reg_1 = re.compile(r'https?://(?:[-\w.]|(?:%[\da-fA-F]{2}))+', flags=re.M)
            hosts = re.match(reg_1, url).group()
            # 如果链接的host是B站的就换成  “㉡网页链接”
            if hosts in ["https://t.bilibili.com", "https://live.bilibili.com",
                         "https://www.bilibili.com", "https://space.bilibili.com",
                         "https://m.bilibili.com", "https://mall.bilibili.com",
                         "https://b23.tv", "https://manga.bilibili.com"]:
                description = description.replace(url, self.rich_text_Instead[1] + "网页链接")
        info_list.append({"text_type": "link", "origin_text": self.rich_text_Instead[1] + "网页链接"})
        # endregion
        return {"content": description, "heterochrosis_info_list": info_list, "emoji_img_list": emoji_list}

    async def get_particular_text(self, description=None, at_control=None):
        """
        获取所有蓝色的文本
        :param description: 文本主体
        :param at_control: ctrl列表
        :return:
        """
        info_list = []
        emoji_list = []
        # 取出所有的ctrl字符
        if at_control:
            # 过滤掉所有的emoji以防止其造成ctrl错位
            content = await self.filter_emoji(description)
            for at_detail in at_control:
                at_location = at_detail.location
                at_length = at_detail.length
                at_type = at_detail.type
                if at_type == 1:
                    at_start = at_location
                    at_end = at_start + at_length
                    at_content = content[at_start:at_end]
                    info_list.append({"origin_text": at_content, "at_type": 1, "text_type": "at"})
                elif at_type == 3:
                    at_start = at_detail.location
                    at_end = at_start + int(at_detail.data)
                    at_content = content[at_start:at_end]
                    info_list.append(
                        {"origin_text": at_content, "new_content": self.rich_text_Instead[5] + at_content,
                         "at_type": at_type,
                         "text_type": "at"})

                else:
                    at_start = at_detail.location
                    at_end = at_start + int(at_detail.data)
                    at_content = content[at_start:at_end]
                    info_list.append(
                        {"origin_text": at_content, "new_content": self.rich_text_Instead[at_type] + at_content,
                         "at_type": at_type,
                         "text_type": "at"})

            for info in info_list:
                if info["at_type"] != 1:
                    description = description.replace(info["origin_text"], info["new_content"])

        # 取出所有富文本字符并且把原字符换成富文本字符
        if self.dynamic.display and self.dynamic.display.rich_text:
            for rich_detail in self.dynamic.display.rich_text.rich_details:
                if rich_detail.icon_type == 1:
                    description = description.replace(rich_detail.orig_text,
                                                      self.rich_text_Instead[0] + rich_detail.text)
                    info_list.append({"origin_text": rich_detail.text, "text_type": "rich_text"})
                else:
                    description = description.replace(rich_detail.orig_text,
                                                      self.rich_text_Instead[3] + rich_detail.text)
                    info_list.append({"origin_text": rich_detail.text, "text_type": "rich_text"})
        # 把所有的bili_emoji换成特殊字符，并且获取所有的bili_emoji图片
        if self.dynamic.display and self.dynamic.display.emoji_info:
            emoji_url_list = []
            for i in range(len(self.dynamic.display.emoji_info.emoji_details)):
                emoji_details = self.dynamic.display.emoji_info.emoji_details
                description = description.replace(emoji_details[i].emoji_name, self.emoji_pic_tag_instead[i])
                info_list.append(
                    {self.emoji_pic_tag_instead[i]: emoji_details[i].emoji_name, "text_type": "bili_emoji"})
                emoji_url_list.append({"url": emoji_details[i].url, "name": emoji_details[i].emoji_name})
            with ThreadPoolExecutor(max_workers=5) as pool:
                results = pool.map(self.get_emoji, emoji_url_list)
                for i in results:
                    emoji_list.append(i)
        # 获取所有的话题标签
        if self.dynamic.display and self.dynamic.display.topic_info:
            if self.dynamic.display.topic_info.topic_details:
                for topic_detail in self.dynamic.display.topic_info.topic_details:
                    info_list.append({"origin_text": f"#{topic_detail.topic_name}#", "text_type": "topic"})
            if self.dynamic.display.topic_info.new_topic:
                topic_name = self.dynamic.display.topic_info.new_topic.name
                description = self.rich_text_Instead[6] + topic_name + "\n" + description
                info_list.append({"origin_text": self.rich_text_Instead[6] + topic_name, "text_type": "new_topic"})

        # 去除文本主体中的特殊符号
        description = description.replace("\r", "")
        if self.emoji_list:
            temp = []
            for i in self.emoji_list:
                if len(i) != 1:
                    emoji_first = i[0]
                    description = description.replace(i, emoji_first)
                    temp.append(emoji_first)
                    continue
                temp.append(i)
            self.emoji_list = temp
        # region
        # 获取文本中所有的url
        reg = r'https?:[0-9a-zA-Z.?#&=_@(-/\d]+'
        urls = re.findall(reg, description)
        for url in urls:
            reg_1 = re.compile(r'https?://(?:[-\w.]|(?:%[\da-fA-F]{2}))+', flags=re.M)
            hosts = re.match(reg_1, url).group()
            # 如果链接的host是B站的就换成  “㉡网页链接”
            if hosts in ["https://t.bilibili.com", "https://live.bilibili.com",
                         "https://www.bilibili.com", "https://space.bilibili.com",
                         "https://m.bilibili.com", "https://mall.bilibili.com",
                         "https://b23.tv", "https://manga.bilibili.com"]:
                description = description.replace(url, self.rich_text_Instead[1] + "网页链接")
        info_list.append({"text_type": "link", "origin_text": self.rich_text_Instead[1] + "网页链接"})
        # endregion
        return {"content": description, "heterochrosis_info_list": info_list, "emoji_img_list": emoji_list}

    async def calculate_particular_text_index(self, particular_text_info, description):
        """
        计算所有的蓝色文字的索引
        :param particular_text_info_dict: 蓝色字体的具体信息字典
        :param description: 主体文本
        :return:
        """
        # 先使用元组，后面方便合并，在最后会转成集合
        particular_text_index_set = ()
        # 如果特殊文本是at别人
        for particular_text_info_dict in particular_text_info:
            if particular_text_info_dict["text_type"] == "at":
                for t in re.finditer(particular_text_info_dict["origin_text"], description):
                    particular_text_index_set = particular_text_index_set + tuple(x for x in range(t.start(), t.end()))
            elif particular_text_info_dict["text_type"] == "rich_text" or particular_text_info_dict[
                "text_type"] == "topic":
                # 加号和左括号会影响正则，把它们换成转义字符
                origin_text = particular_text_info_dict["origin_text"].replace("+", "\+").replace("(", "\(")
                for t in re.finditer(origin_text, description):
                    particular_text_index_set = particular_text_index_set + tuple(x for x in range(t.start(), t.end()))
            elif particular_text_info_dict["text_type"] == "link":
                for t in re.finditer(particular_text_info_dict["origin_text"], description):
                    particular_text_index_set = particular_text_index_set + tuple(
                        x for x in range(t.start() + 1, t.end()))
            elif particular_text_info_dict["text_type"] == "new_topic":
                for t in re.finditer(particular_text_info_dict["origin_text"], description):
                    particular_text_index_set = particular_text_index_set + tuple(x for x in range(t.start(), t.end()))

        return set(particular_text_index_set)

    async def calculate_position(self, start_x: int, start_y: int, x_restrain: int, content: str, line_height: int,
                                 font_size: int, particular_text_index_set=None, img_list=None, font_color=None):
        tag_pic_list = {"0": "play.png", "1": "link.png", "2": "lottery.png", "3": "article.png", "4": "taobao.png",
                        "5": "icon_vote.png", "6": "new_topic.png"}
        position_list = []
        x, y = start_x, start_y

        for i in range(len(content)):
            while 1:
                # 如果是图标
                if content[i] in self.rich_text_Instead:
                    # 打开图标图片
                    t_content = Image.open(os.path.join(self.__base_path, "Picture", tag_pic_list[
                        str(self.rich_text_Instead.index(content[i]))])).convert(
                        "RGBA").resize((font_size, font_size), Image.ANTIALIAS)
                    position_list.append({"info_type": "img", "content": t_content, "position": (x, y + 3)})
                    x += font_size + 2
                    if x > x_restrain:
                        x = start_x
                        y += line_height
                    break
                # 如果是bili_emoji
                if content[i] in self.emoji_pic_tag_instead:
                    t_content = img_list[self.emoji_pic_tag_instead.index(content[i])].resize((font_size, font_size),
                                                                                              Image.ANTIALIAS)
                    position_list.append({"info_type": "img", "content": t_content, "position": (x, y + 5)})
                    x += font_size
                    if x > x_restrain:
                        x = start_x
                        y += line_height
                    break
                # 如果是emoji
                if await self.isEmoji(content[i]):
                    # 获取emoji大小
                    size = self.emoji_font.getsize(content[i])
                    # 新建一个图片来承载emoji
                    img = Image.new("RGBA", size)
                    draw = ImageDraw.Draw(img)
                    # 写入emoji
                    draw.text(xy=(0, 0), text=content[i], font=self.emoji_font, embedded_color=True)
                    # 把emoji图片缩放到文字字体的大小
                    img = img.resize((font_size, font_size), Image.ANTIALIAS)
                    # 将文字信息放入字典
                    position_list.append({"info_type": "img", "content": img, "position": (x, y)})
                    x += font_size
                    if x > x_restrain:
                        x = start_x
                        y += line_height
                    break
                # 如果是换行
                if content[i] == "\n":
                    x = start_x
                    y += line_height
                    break
                # 如果是文字
                size = self.text_font.getsize(content[i])
                # 如果是蓝色字体
                if i in particular_text_index_set:
                    position_list.append(
                        {"info_type": "text", "content": content[i], "position": (x, y), "font_color": "#00A0D8",
                         "font_size": font_size})
                else:
                    position_list.append(
                        {"info_type": "text", "content": content[i], "position": (x, y), "font_color": font_color,
                         "font_size": font_size})
                if ord(content[i]) not in self.font_key:
                    x += self.standby_font.getsize(content[i])[0]
                else:
                    x += size[0]
                if x > x_restrain:
                    x = start_x
                    y += line_height
                break
        return position_list

    async def filter_emoji(self, content):
        """
        将文本主体的emoji过滤掉
        :param content: 文本主体
        :return:
        """
        try:
            cont = re.compile(u'[\U00010000-\U0010ffff]')
        except re.error:
            cont = re.compile(u'[\uD800-\uDBFF][\uDC00-\uDFFF]')
        return cont.sub(u'32', content)

    def get_emoji(self, emoji_info):
        emoji_name = emoji_info["name"]
        emoji_cache_path = os.path.join(self.Emoji_cache_path, emoji_name + ".png")
        if os.path.exists(emoji_cache_path):
            img = Image.open(emoji_cache_path).convert("RGBA")
        else:
            response = httpx.get(emoji_info["url"])
            with open(emoji_cache_path, "wb") as f:
                f.write(response.content)
            img = Image.open(BytesIO(response.content)).convert("RGBA")
        return img

    def draw_pic(self, info):
        if info["info_type"] == "text":
            if ord(info["content"]) not in self.font_key:
                self.draw.text(info["position"], info["content"], fill=info["font_color"], font=self.standby_font)
            else:
                self.draw.text(info["position"], info["content"], fill=info["font_color"], font=self.text_font)
        elif info["info_type"] == "emoji":

            self.draw.text(xy=info["position"], text=info["content"], font=self.emoji_font, embedded_color=True)
        else:
            self.content_img.paste(info["content"], info["position"], info["content"])

    async def isEmoji(self, content):
        if not content:
            return False
        if content in self.emoji_list:
            return True
        else:
            return False
